using System;
using System.Collections.Generic;
using System.ServiceModel;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Http.Features;
using Microsoft.AspNetCore.Rewrite;
using Microsoft.AspNetCore.StaticFiles;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.FileProviders;
using Microsoft.Extensions.Hosting;
using MySystem.PublicClass.GraphQL;
using System.Text;
using Microsoft.IdentityModel.Tokens;
using System.Linq;

namespace MySystem
{
    public class Startup
    {
        public Startup(IConfiguration configuration)
        {
            Configuration = configuration;

        }

        public IConfiguration Configuration { get; }

        // This method gets called by the runtime. Use this method to add services to the container.
        public void ConfigureServices(IServiceCollection services)
        {
            services.AddControllersWithViews();
            services.AddRouting(options =>
            {
                options.LowercaseUrls = true;
            });
            services.AddSingleton<IHttpContextAccessor, HttpContextAccessor>();
            services.Configure<Setting>(Configuration.GetSection("Setting"));
            services.AddCors(option => option.AddPolicy("cors", policy => policy.AllowAnyHeader().AllowAnyMethod().AllowCredentials().SetIsOriginAllowed(_ => true)));
            services.AddMvc(options =>
            {
                options.EnableEndpointRouting = false;
                options.Filters.Add(typeof(GlobalExceptions));
            });
            services.AddSession(options =>
            {
                // 设置 Session 过期时间
                options.IdleTimeout = TimeSpan.FromHours(1);
                options.Cookie.HttpOnly = true;
            });
            services.AddSingleton<IRepository, Repository>();
            services.Configure<FormOptions>(x =>
            {
                x.MultipartBodyLengthLimit = 50 * 1024 * 1024;//不到300M
            });
            //生成密钥
            var symmetricKeyAsBase64 = Configuration["Setting:JwtSecret"];
            var keyByteArray = Encoding.ASCII.GetBytes(symmetricKeyAsBase64);
            var signingKey = new SymmetricSecurityKey(keyByteArray);
            //认证参数
            services.AddAuthentication("Bearer").AddJwtBearer(o =>
            {
                o.TokenValidationParameters = new TokenValidationParameters
                {
                    ValidateIssuerSigningKey = true,//是否验证签名,不验证的画可以篡改数据，不安全
                    IssuerSigningKey = signingKey,//解密的密钥
                    ValidateIssuer = true,//是否验证发行人，就是验证载荷中的Iss是否对应ValidIssuer参数
                    // ValidIssuer = Configuration["Setting:JwtIss"],//发行人
                    IssuerValidator = (m, n, z) =>
                    {
                        return n.Issuer;
                    },
                    ValidateAudience = true,//是否验证订阅人，就是验证载荷中的Aud是否对应ValidAudience参数
                    // ValidAudience = Configuration["Setting:JwtAud"],//订阅人
                    AudienceValidator = (m, n, z) =>
                    {
                        string check = RedisDbconn.Instance.Get<string>("utoken:" + n.Issuer);
                        return m != null && m.FirstOrDefault().Equals(check);
                    },
                    ValidateLifetime = true,//是否验证过期时间，过期了就拒绝访问
                    ClockSkew = TimeSpan.Zero,//这个是缓冲过期时间，也就是说，即使我们配置了过期时间，这里也要考虑进去，过期时间+缓冲，默认好像是7分钟，你可以直接设置为0
                    RequireExpirationTime = true,
                };
            });

            // services.AddHostedService<TradeService>();
            // services.AddHostedService<TeamTopService>();
            // services.AddHostedService<PopService>();
            // services.AddHostedService<SycnSpTimer>(); //同步SP数据

            // services.AddHostedService<TimerStatTimer>(); //实时统计创客、激活商户数
            services.AddHostedService<PosTradeStatTimer>(); // 统计头天的交易额、商户型创客、激活奖励、开机奖励
            services.AddHostedService<AlipayPayBack2Timer>(); //支付宝回调处理

            //services.AddHttpContextAccessor();

            string appkey = Configuration["Setting:AppKey"];
            string appid = Configuration["Setting:AppId"];
            string checkurl = Configuration["Setting:CheckUrl"];
            string serviceurl = Configuration["Setting:WebServiceUrl"];
            string schemeurl = Configuration["Setting:DbSchemeUrl"];
            MySystemLib.SystemPublicFuction.appkey = appkey;
            MySystemLib.SystemPublicFuction.appid = appid;
            MySystemLib.SystemPublicFuction.checkurl = checkurl;
            MySystemLib.SystemPublicFuction.appcheck = "success";
            Dictionary<string, Dictionary<string, string>> tables = new Dictionary<string, Dictionary<string, string>>();
            System.Data.DataTable tablecollection = Library.dbconn.dtable("select DISTINCT TABLE_NAME from information_schema.columns where table_schema = 'KxsMainServer'");
            foreach (System.Data.DataRow subtable in tablecollection.Rows)
            {
                Dictionary<string, string> Columns = new Dictionary<string, string>();
                System.Data.DataTable columncollection = Library.dbconn.dtable("select COLUMN_NAME,DATA_TYPE from information_schema.columns where table_schema = 'KxsMainServer' and TABLE_NAME='" + subtable["TABLE_NAME"].ToString() + "'");
                foreach (System.Data.DataRow column in columncollection.Rows)
                {
                    string datatype = column["DATA_TYPE"].ToString();
                    if (datatype == "decimal")
                    {
                        datatype = "numeric";
                    }
                    Columns.Add(column["COLUMN_NAME"].ToString(), datatype);
                }
                tables.Add(subtable["TABLE_NAME"].ToString(), Columns);
            }
            MySystemLib.SystemPublicFuction.dbtables = tables;

            RedisDbconn.csredis = new CSRedis.CSRedisClient(Configuration["Setting:RedisConnStr"]);
        }

        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        public void Configure(IApplicationBuilder app, IWebHostEnvironment env)
        {
            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
                // app.UseExceptionHandler("/Home/Error");
            }
            else
            {
                app.UseExceptionHandler("/Home/Error");
                app.UseHsts();
            }
            Library.function.WritePage("/", "WebRootPath.txt", env.WebRootPath);
            // app.UseStatusCodePagesWithReExecute("/public/errpage/pc/{0}.html");

            // RequestDelegate handler = async context =>
            // {
            //     var response = context.Response;
            //     if (response.StatusCode < 500)
            //     {
            //         response.("/public/errpage/pc/{0}.html");
            //     }
            // };
            // app.UseStatusCodePages(builder => builder.Run(handler));
            app.UseStaticFiles();
            // app.UseStaticFiles(new StaticFileOptions
            // {
            //     FileProvider = new PhysicalFileProvider(AppContext.BaseDirectory + "/static"),
            //     RequestPath = "/static"
            // });
            // app.UseStaticFiles(new StaticFileOptions
            // {
            //     FileProvider = new PhysicalFileProvider(AppContext.BaseDirectory + "/" + Configuration["Setting:Database"]),
            //     RequestPath = "/" + Configuration["Setting:Database"]
            // });
            app.UseStaticFiles(new StaticFileOptions
            {
                ContentTypeProvider = new FileExtensionContentTypeProvider(new Dictionary<string, string>
                {
                      { ".apk", "application/vnd.android.package-archive" }
                })
            });

            app.UseCors("cors");
            app.UseAuthentication();
            app.UseRouting();
            app.UseAuthorization();
            app.UseSession();

            app.UseEndpoints(endpoints =>
            {
                endpoints.MapControllerRoute(
                    name: "default",
                    pattern: "{controller=Home}/{action=Index}/{Id?}");
            });

            //必须打开的
            ActiveRewardTimer.Instance.Start(); //实时处理激活记录
            ChangePosTimer.Instance.Start(); //售后换新执行机具数据转移
            SycnSpBindService.Instance.Start(); //同步SP绑定数据
            SycnSpMerchantService.Instance.Start(); //同步SP商户数据
            SycnSpActiveService.Instance.Start(); //同步SP激活数据
            SycnSpTradeService.Instance.Start(); //同步SP交易数据
            SycnSpChangeBindService.Instance.Start(); //同步SP换绑数据
            SycnSpUnBindService.Instance.Start(); //同步SP解绑数据
            RedPackageV2Helper.Instance.Start(); //每天生成红包
            RedPackageV2Helper.Instance.StartStatTop10(); //红包活动统计排行
            RedPackageV2Helper.Instance.StartSendPrize(); //红包活动发奖
            BalancePayBackService.Instance.Start(); //余额支付队列
            ReservePayBackService.Instance.Start(); //储备金支付队列
            LeaderPrizeService.Instance.Start(); //大盟主奖励发奖
            MessageCenterService.Instance.Start(); // 消息队列
            RecommendActStatService.Instance.Start(); //推荐王奖励数据统计
            StoreApplyHelper.Instance.Start(); // 每月1号重置仓库额度
            StoreApplyHelper.Instance.StartEverTime();
            StatService.Instance.StatUserLevel(); //升级
            ProfitHelperV2.Instance.StatProfit(); //创客分润
            //必须打开的

            HelpProfitPreMerchantHelper.Instance.Start(); //助利宝每天增加指定数量商机
            SetDepositService.Instance.Start(); //调整费率（通知、标记）
            SetDepositPostService.Instance.Start(); //提交支付公司设置费率接口
            SetDepositPostService.Instance.StartKdb(); //监控开店宝费率设置结果
            SetFeeFlagService.Instance.Start(); //178天提前通知创客费率调升消息
            ResetSmallStoreHelper.Instance.Start(); //每月重置小分仓额度

            // StatService.Instance.StartPosActNum(); //实时统计激活数
            // StatService.Instance.StartNewUserNum(); //实时统计新增创客数
            // StatService.Instance.StatProfit(); //实时统计创客收益
            // StatServiceTmp.Instance.Start();
            // StatService.Instance.StartEverDay("");
            // StatService.Instance.StartEverDayV2();
            // RedPackageHelper.Instance.Start();
            // ProfitHelperV2.Instance.StatProfit(); //统计分润
            // StatService.Instance.ListenFluxRecord();
            // StatService.Instance.StartEverDay2();
            // TestHelper.Instance.Start();
            // TestService.Instance.Start();
            // StatService.Instance.StartEverDay();


            // 备用，暂时不放开的
            // StatStoreDataService.Instance.Start();
            // RabbitMQClient.Instance.StartReceive("SycnTableData");
            // PayHelper.Instance.Start();
            // OrderHelper.Instance.Start();
            // OrderRefundHelper.Instance.Start();
            // ProductCommentHelper.Instance.Start();
            // TimedTaskHelper.Instance.Start();
            // TaskFlowHelper.Instance.Start();
        }
    }
}